package gov.fema.web.srce;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.naming.Context;
import javax.naming.InitialContext;

import org.apache.log4j.Logger;

import gov.fema.util.CommonDataLoader;
import gov.fema.util.email.Email;
import gov.fema.util.email.EmailModel;


/**
 * BaseManager provides base database connectivity for the SRCE application
 * 
 * @author      Alla Sharkova
 */
public class BaseManager {

    protected static final Logger logger = Logger.getLogger(BaseManager.class);

    protected Properties props;
    protected static long refreshTime = 0;
	
	protected SimpleDateFormat formatter = null;


    static {
		// initialize the CommonDataLoader, since we need access to State and Disaster information
        logger.warn("STARTUP: Initializing the CommonDataLoader ...");
    	try {
	    	CommonDataLoader cdl = new CommonDataLoader();
	        cdl.initialize();
	        logger.warn("STARTUP: CommonDataLoader initialized");
    	}
        catch (Exception e) {
            logger.error("STARTUP: CommonDataLoader initialization failed", e);
        }
    }


    /**
	 * Default Constructor.
	 *
	 */
	public BaseManager() {
		formatter = new SimpleDateFormat("MM/dd/yyyy");

		try  {
		    Context initCtx = new InitialContext();
		    Context envCtx = (Context) initCtx.lookup("java:comp/env");
			String fileConfig = (String)envCtx.lookup("fileConfig");

			props = new Properties();
			if (fileConfig != null) {
			    // read config file
			    FileInputStream fis = new FileInputStream(fileConfig);
			    BufferedInputStream bis = new BufferedInputStream(fis);
				props.load(bis);
				bis.close();
				fis.close();

				if (refreshTime == 0) {
				    String rtime = props.getProperty("refreshtime").trim();
		            if (rtime != null) 
		    		    refreshTime = (Integer.parseInt(rtime))*60*1000;
				}
			}
		}
		catch (Exception e) {
		    logger.error("ERROR in BaseManager()", e);
		}
	}
	
	/**
	 * Get db connection.
	 *
	 * @return		Connection object 
	 * @throws		Exception
	 */
	protected static Connection getConnection() throws Exception {
	    return PoolManager.getConnection();
	}

    /**
     * Gets the value of refreshTime
     *
     * @return the value of refreshTime
     */
    public final long getRefreshTime() {
        return refreshTime;
    }

    /**
     * Sets the value of refreshTime
     *
     * @param argRefreshTime Value to assign to this.refreshTime
     */
    public final void setRefreshTime(final int argRefreshTime) {
        refreshTime = argRefreshTime;
    }

    /**
	 * Send email.
	 *
	 * @param   	smtpServer			smtp server
	 * @param   	smtpPort			smtp port
	 * @param   	errorEmailTo		To email addresses
	 * @param   	errorEmailFrom		From email address
	 * @param   	subject				email subject
	 * @param   	message				email message
	 * @return		boolean, true if email was ent successfully
	 */
	protected boolean sendEmail(	String smtpServer, String smtpPort, 
    								String errorEmailTo, String errorEmailFrom, 
                                    String subject, String message) {
		boolean sent = false;

		StringTokenizer st = new StringTokenizer(errorEmailTo, ";");
        int count = st.countTokens();
		String[] recipients = new String[count];
		for (int i = 0; i < count; i++) {
			recipients[i] = st.nextToken();
		}

		if (recipients != null && recipients.length > 0) {
			EmailModel em = new EmailModel();
			em.setSMTPServer(smtpServer);
			em.setSMTPPort(smtpPort);
			em.setTo(recipients);
			em.setFrom(errorEmailFrom);
            
            em.setMsg(message);
			em.setSubject(subject);
			sent = Email.sendSMTPEmail(em);
		}

	    return sent;
	}									
	
	/**
	 * Get code description.
	 *
	 * @param   	v				vector of String[2]
	 * @param   	str				code
	 * @param   	codeIndex		index of code
	 * @param   	descrIndex		index of description
	 * @return		code description
	 */
	public static String getCodeDescr(	Vector v, String str, 
    									int codeIndex, int descrIndex) {
		if (v != null) {
	    	for (int i = 0; i < v.size(); i++) {
	        	if (((String[])v.elementAt(i))[codeIndex].equals(str))
	            	return ((String[])v.elementAt(i))[descrIndex];
	    	}
		}
	    return "";
	} 
	
	/**
	 * Get code description.
	 *
	 * @param   	v				vector of String[2]
	 * @param   	str				code
	 * @return		code description
	 */
	public static String getCodeDescr(	Vector v, String str) {
		return getCodeDescr(v, str, 0, 1);
	} 

	
}

