/*
 * Created on Aug 5, 2003
 *
 */
package gov.fema.web.srce;

import gov.fema.web.srce.objects.Building;
import gov.fema.web.srce.objects.DecisionTree;
import gov.fema.web.srce.util.DbUtil;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;


/**
 *
 * This class manages all the database accesses related to building information. 
 * It is modeled after the TravelManager class I recieved from FEMA
 * 
 * The database contains 3 building-related tables: 
 * 		buildings   		- all the data from the spreadsheets. Each record is a different building. 
 * 		smallTreeIntersect 	- Represents the relationship between internal nodes and the leaves in the small tree. 
 * 		largeTreeIntersect	- Represents the relationship between internal nodes and the leaves in the large tree.
 * 
 *@author n-chilson
 *
 **/
public class BuildingManager extends BaseManager {

	protected static final String stmtGetBuildingText = "SELECT * FROM buildings WHERE id = ?";
	protected static final String stmtGetBuildingsForSmallTreeTermNode = "SELECT * FROM buildings WHERE small_tree_node = ? ORDER BY id ASC";
	protected static final String stmtGetBuildingsForLargeTreeTermNode = "SELECT * FROM buildings WHERE large_tree_node = ? ORDER BY id ASC";
	protected static final String stmtGetBuildingsForSmallTreeInternalNode = "SELECT * FROM buildings, smallTreeIntersect WHERE smallTreeIntersect.internalNodeId=? AND buildings.small_tree_node = smallTreeIntersect.terminalNodeId ORDER BY buildings.id ASC";
	protected static final String stmtGetBuildingsForLargeTreeInternalNode = "SELECT * FROM buildings, largeTreeIntersect WHERE largeTreeIntersect.internalNodeId=? AND buildings.large_tree_node = largeTreeIntersect.terminalNodeId ORDER BY buildings.id ASC";

	protected static final Logger logger = org.apache.log4j.Logger.getLogger(BuildingManager.class);
	
	private Building building;
	private Vector buildings;
	
	/**
	 * Default constructor
	 */
	public BuildingManager() {
		super();
		building = null;
	}
		
	/**
	 * Load current Building object from database.
	 *
	 * @param 	id		the id of the building we want to retrieve.
	 * @return 	String	an error string. Empty if no errors.
	 */
	public String loadBuilding(	long id ) 
	{
		Connection conn = null;
		ResultSet rs = null;
		PreparedStatement pstmt = null;
		String error = "";
		
		logger.debug("loading building #" + id);
		
		try {
			conn = getConnection();

			logger.debug("About to execute : " + stmtGetBuildingText);
			pstmt = conn.prepareStatement(stmtGetBuildingText);
			pstmt.setLong(1, id);
			rs = pstmt.executeQuery();
        	
			if (rs.next()) {
				building = setBuildingValues(rs);
			}
			else {
				error = "Building not found.";
			}
		}
		catch(SQLException e) {
			logger.error(e);
			error = "SQL ERROR in loadBuilding(): " + e.getMessage();
		}
		catch(Exception ex) {
			logger.error(ex);
			error = "ERROR in loadBuilding(): " + ex.getMessage();
		}
		finally {
			DbUtil.cleanup(conn, pstmt, rs);
		}
    
		return error;
	}
	
	/**
	 * Create a Building object from a ResultSet.
	 *
	 * @param rs		ResultSet
	 * @return		TravelEntry
	 */
	protected Building setBuildingValues(	ResultSet rs) 
											throws java.sql.SQLException {
		Building b = new Building(	rs.getInt("ID"),
									rs.getInt("SMALL_TREE_NODE"),
									rs.getInt("LARGE_TREE_NODE"),
									StringUtils.trimToEmpty(rs.getString("CONTRIB")),		
									formatter.format(rs.getDate("SUB_DATE")),
									StringUtils.trimToEmpty(rs.getString("COUNTY")),
									StringUtils.trimToEmpty(rs.getString("STATE")),
									rs.getInt("ZONE"), 
									StringUtils.trimToEmpty(rs.getString("SOIL")),
									rs.getInt("ABOVE_STORIES"), 
									rs.getInt("BELOW_STORIES"), 
									rs.getDouble("AREA"),
									StringUtils.trimToEmpty(rs.getString("AGE")),
									StringUtils.trimToEmpty(rs.getString("MODEL")),
									StringUtils.trimToEmpty(rs.getString("HISTORIC")),
									StringUtils.trimToEmpty(rs.getString("BASE_YEAR")),
									rs.getDouble("C_TOTAL"),
									StringUtils.trimToEmpty(rs.getString("SOURCE")),
									StringUtils.trimToEmpty(rs.getString("SCOPE")),
									rs.getDouble("ADD_SPACE"),
									StringUtils.trimToEmpty(rs.getString("P_OBJ")),
									StringUtils.trimToEmpty(rs.getString("REHAB1")),
									StringUtils.trimToEmpty(rs.getString("REHAB2")),
									StringUtils.trimToEmpty(rs.getString("REHAB3")),
									StringUtils.trimToEmpty(rs.getString("REHAB4")),
									StringUtils.trimToEmpty(rs.getString("REHAB5")),
									StringUtils.trimToEmpty(rs.getString("REHAB6")),
									StringUtils.trimToEmpty(rs.getString("ASBESTOS")),
									StringUtils.trimToEmpty(rs.getString("DISABLED")),
									StringUtils.trimToEmpty(rs.getString("SYS_IMP")),
									StringUtils.trimToEmpty(rs.getString("REPAIR")),
									StringUtils.trimToEmpty(rs.getString("OCCUP_COND")),
									StringUtils.trimToEmpty(rs.getString("STRUCTURE")),
									StringUtils.trimToEmpty(rs.getString("EXT_FALLING")),
									StringUtils.trimToEmpty(rs.getString("SELECTED_INTS")),
									StringUtils.trimToEmpty(rs.getString("ALL_INTS")),
									rs.getDouble("C_STRUCT"),
									StringUtils.trimToEmpty(rs.getString("UNCERT_AREAS")),
									StringUtils.trimToEmpty(rs.getString("UNCERT_TOTAL")),
									StringUtils.trimToEmpty(rs.getString("UNCERT_STRUCT")),
									rs.getDouble("C_RELOCATION"),
									rs.getDouble("C_A_E_FEES"),
									rs.getDouble("C_PROJ_MGR"),
									rs.getInt("DURATION"),
									rs.getDouble("C_REPAIR"),
									rs.getDouble("C_ASBESTOS"),
									rs.getDouble("C_DISABLED"),
									rs.getDouble("C_SYS_IMP"),
									rs.getDouble("C_NONSTRUCT"),
									StringUtils.trimToEmpty(rs.getString("PLAN")),
									rs.getDouble("BASE_X"),
									rs.getDouble("BASE_Y"),
									rs.getDouble("FLOOR_X"),
									rs.getDouble("FLOOR_Y"),
									rs.getDouble("STORY_H"),
									rs.getDouble("TOTAL_H"),
									StringUtils.trimToEmpty(rs.getString("ROOFFRAME")),
									StringUtils.trimToEmpty(rs.getString("FLOORFRAME")),
									StringUtils.trimToEmpty(rs.getString("DIAPHRAGM")),
									StringUtils.trimToEmpty(rs.getString("CLADDING")),
									StringUtils.trimToEmpty(rs.getString("SETTLING")),
									StringUtils.trimToEmpty(rs.getString("CONDITION")),
									StringUtils.trimToEmpty(rs.getString("COLUMN_TYPE")),
									StringUtils.trimToEmpty(rs.getString("BEARING_WALLS")),
									StringUtils.trimToEmpty(rs.getString("FOUNDATIONS")),
									StringUtils.trimToEmpty(rs.getString("LONG_LATERAL")),
									StringUtils.trimToEmpty(rs.getString("TRAN_LATERAL")),
									rs.getDouble("COST_PER_SF"),
									rs.getDouble("TIMEFACTOR"),
									rs.getDouble("STATEFACTOR"),
									rs.getDouble("TOTALFACTOR"),
									rs.getDouble("F_C_TOTAL"),
									rs.getDouble("F_C_STRUCT"),
									rs.getDouble("F_C_RELOCATION"),
									rs.getDouble("F_C_A_E_FEES"),
									rs.getDouble("F_C_PROJ_MGR"),
									rs.getDouble("F_C_REPAIR"),
									rs.getDouble("F_C_ASBESTOS"),
									rs.getDouble("F_C_DISABLED"),
									rs.getDouble("F_C_SYS_IMP"),
									rs.getDouble("F_C_NONSTRUCT"),
									rs.getDouble("QUALITYFACTOR"),
									StringUtils.trimToEmpty(rs.getString("CODE_USED")),
									StringUtils.trimToEmpty(rs.getString("SPECIAL")),
									StringUtils.trimToEmpty(rs.getString("REHAB"))
									);
		return b;
	}
	
	/**
	 * 
	 * Executes a SQL query and creates a collection of Building objects out of 
	 * the result set.
	 * 
	 * @param query	The query to execute.
	 * @param id	Id of the node
	 * @return		An error String. Empty if no errors.
	 */
	private String loadBuildings(String query, long id)
	{
		String error = "";
		buildings = new Vector();

		Connection conn = null;
		ResultSet rs = null;
		PreparedStatement pstmt = null;

		try {
			conn = getConnection();

			logger.debug("About to execute : " + query);
			pstmt = conn.prepareStatement(query);
			pstmt.setLong(1, id);
			rs = pstmt.executeQuery();
	
			logger.debug("Number of returned entries:" + rs.toString());
	
			if(rs.next())
			{
				logger.debug("At least one record.");
		
				do{
					buildings.add( setBuildingValues(rs) );
					logger.debug("Added record.");
				} while (rs.next());
			}
			else
			{
				error = "ERROR: No buildings found using query " + query;
			}
	
			logger.debug("Added " + buildings.size() + " buildings");
		}
		catch(SQLException e) {
			logger.error(e);
			error = "SQL ERROR in loadBuildings(): " + e.getMessage();
		}
		catch(Exception ex) {
			logger.error(ex);
			error = "ERROR in loadBuildings(): " + ex.getMessage();
		}
		finally {
			DbUtil.cleanup(conn, pstmt, rs);
		}

		return error;
	}
	
	/**
	 * 
	 * @param method	which tree is being used
	 * @param nodeId	the terminal node for which buildings should be loaded
	 * @return		error string. Empty if no error.
	 */
	public String loadBuildingsForTerminalNode(String method, long nodeId )
	{
		if (method.equals(DecisionTree.METHOD_1))
		{
			return loadBuildingsForSmallTreeTerminalNode( nodeId );
		}
		else if (method.equals(DecisionTree.METHOD_2))
		{
			return loadBuildingsForLargeTreeTerminalNode( nodeId );
		}
		else
		{
			return  "ERROR: unknown tree method in loadBuildingsForTerminalNode() ";
		}
	}
	
	/**
	 * @param method	which tree is being used
	 * @param nodeId	the terminal node for which buildings should be loaded
	 *
	 * @return		error string. Empty if no error.
	 */
	public String loadBuildingsForInternalNode(String method, long nodeId )
	{
		if (method.equals(DecisionTree.METHOD_1))
		{
			return loadBuildingsForSmallTreeInternalNode( nodeId );
		}
		else if (method.equals(DecisionTree.METHOD_2))
		{
			return loadBuildingsForLargeTreeInternalNode( nodeId );
		}
		else
		{
			return  "ERROR: unknown tree method in loadBuildingsForTerminalNode() ";
		}
	}
	
	/**
	 * 
	 * Loads a small tree terminal node's buildings.
	 * 
	 * @param 	nodeId	the id for the node whose buildings we want to load.
	 * @return			An error String. Empty if no errors.
	 */
	public String loadBuildingsForSmallTreeTerminalNode( long nodeId )
	{
		//The DB uses a different ID for terminal nodes than the decisionTree does.
		//Convert the nodeID from the raw nodeID(1-39) to a terminalNodeID(1-20)
		long terminalNodeID = nodeId - 19;
		
		String error = loadBuildings(stmtGetBuildingsForSmallTreeTermNode, terminalNodeID);
		
		return error;
	}
	
	/**
	 * 
	 * Loads a large tree terminal node's buildings.
	 * 
	 * @param nodeId	the id for the node whose buildings we want to load.
	 * @return			an error String. Empty if no errors.
	 */
	public String loadBuildingsForLargeTreeTerminalNode( long nodeId )
	{
		// The DB uses a different ID for terminal nodes than the decisionTree does.
		// Convert the nodeID from the raw NodeID(1-111) to a terminalNodeID(1-56)
		long terminalNodeID = nodeId - 55;
		
		String error = loadBuildings(stmtGetBuildingsForLargeTreeTermNode, terminalNodeID);
		
		return error;
	}
	
	/**
	 * 
	 * Loads a small tree internal node's buildings.
	 * 
	 * @param nodeId	the id for the node whose buildings we want to load.
	 * @return			an error String. Empty if no errors.
	 */
	public String loadBuildingsForSmallTreeInternalNode( long nodeId )
	{
		// Unlike terminal nodes, internal nodes have the same ID in the DB and the decision tree. 
		// No conversion needed.
		
		String error = loadBuildings(stmtGetBuildingsForSmallTreeInternalNode, nodeId);
	
		return error;
	}
	
	/**
	 * 
	 * Loads a large tree internal node's buildings.
	 * 
	 * @param nodeId	the id for the node whose buildings we want to load.
	 * @return 			an error String. Empty if no errors.
	 */
	public String loadBuildingsForLargeTreeInternalNode( long nodeId )
	{
		//	Unlike terminal nodes, internal nodes have the same ID in the DB and the decision tree. 
		// No conversion needed.
		
		String error = loadBuildings(stmtGetBuildingsForLargeTreeInternalNode, nodeId);

		return error;
	}
		
	/**
	 * 
	 * Once a building is loaded, this method is used to retrieve it.
	 * 
	 * @return	the result of the last single building query
	 */
	public Building getBuilding() {
		return building;
	}

	/**
	 * 
	 * Once a group of building is loaded, this method is used to retrieve them.
	 * 
	 * @return	A Vector containing the results of the 
	 *          last multiple buildings query
	 */
	public Vector getBuildings() {
		return buildings;
	}

}
