/*
 * Created on Dec 17, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package gov.fema.web.srce;

import gov.fema.web.srce.objects.Answer;
import gov.fema.web.srce.objects.Question;
import gov.fema.web.srce.util.DbUtil;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Vector;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 *
 * This class manages all the database accesses for the constant data structures in this
 * application. This includes the states, cities, questions, answers for each type of tree, etc.
 *
 * @author Neil Chilson 
 */
public class ConstantsManager extends BaseManager {

	//protected static final String stmtGetBuildingText = "SELECT * FROM buildings WHERE id = ?";
	protected static final String stmtGetRegionAdjustmentFactors = "SELECT * FROM RegionAdjustmentFactors";
	protected static final String stmtGetQuestions = "SELECT * FROM question";
			
	//protected static final String stmtGetCityAdjustmentFactors = "SELECT * FROM buildings WHERE large_tree_node = ? ORDER BY id ASC";
	//protected static final String stmtGetBuildingsForSmallTreeInternalNode = "SELECT * FROM buildings, smallTreeIntersect WHERE smallTreeIntersect.internalNodeId=? AND buildings.small_tree_node = smallTreeIntersect.terminalNodeId ORDER BY buildings.id ASC";
	//protected static final String stmtGetBuildingsForLargeTreeInternalNode = "SELECT * FROM buildings, largeTreeIntersect WHERE largeTreeIntersect.internalNodeId=? AND buildings.large_tree_node = largeTreeIntersect.terminalNodeId ORDER BY buildings.id ASC";

	protected static final Logger logger = org.apache.log4j.Logger.getLogger(ConstantsManager.class);
	
	public SortedMap states; // contains the multiplicative factors for each state.
	public SortedMap cities; // contains the multiplicative factors for each city.
	public Map questions;
	
	/**
	 * Default constructor
	 */
	public ConstantsManager() {
		super();
		
		states = null;
		cities = null;
		questions = null;
	}

	/**
	 * Load current Building object from database.
	 *
	 * @return	String	an error string. Empty if no errors.
	 */
	public String loadRegionAdjustmentFactors() 
	{
		Connection conn = null;
		ResultSet rs = null;
		Statement stmt = null;
		String error = "";
	
		try {
			conn = getConnection();
			stmt = conn.createStatement();
			String query = stmtGetRegionAdjustmentFactors;
		
			logger.debug("About to execute : " + query);
			rs = stmt.executeQuery(query);
    		
    		logger.debug("Finished query");
    		
			if (rs.next()) {
				logger.debug("Parsing results");
				setRegionAdjustmentFactorsValues(rs);
				logger.debug("Finished parsing results");
			}
			else {
				error = "Region adjustment factors could not be loaded from the database.";
			}
		}
		catch(SQLException e) {
            logger.log(Level.WARN, "loadRegionAdjustmentFactors() Exception", e);
			error = "SQL ERROR in loadRegionAdjustmentFactors(): " + e.getMessage();
		}
		catch(Exception ex) {
            logger.log(Level.WARN, "loadRegionAdjustmentFactors() Exception", ex);
			error = "ERROR in loadRegionAdjustmentFactors(): " + ex.getMessage();
		}
		finally {
			DbUtil.cleanup(conn, stmt, rs);
		}

		if (error != "")
		{
			logger.debug("ERROR: "+ error);
		}

		return error;
	}

	/**
	 * Parse the results of the region adjustment factors
	 * 
	 * @param rs	result set to parse
	 */
	private void setRegionAdjustmentFactorsValues(ResultSet rs) throws SQLException {
		String regionType;
		String regionName;
		Double factor;
		
		states = new TreeMap();
		cities = new TreeMap();
		
		do{
			regionType = StringUtils.trimToEmpty(rs.getString("regionType"));
			regionName = StringUtils.trimToEmpty(rs.getString("regionName"));
			factor = new Double(rs.getDouble("factor"));
			
			logger.debug( "Adding " + regionType + " with factor " + factor.toString());
			
			if ( regionType.equalsIgnoreCase("state") )
			{			
				states.put(regionName, factor);
			}
			else if ( regionType.equalsIgnoreCase("city") )
			{
				cities.put(regionName, factor);
			}
			else
			{
				logger.debug("Unknown regionType '" + regionType + "' in region table.");
			}
			
		} while (rs.next());
		
	}
	
	/**
	 * A gettor using lazy initialization - will create the list if it doesn't exist yet.
	 * 
	 * @return 	a Map of the cities, sorted alphabetically by name
	 */
	public SortedMap getCityFactors() {
		if (cities == null)
		{
			loadRegionAdjustmentFactors();
		}
		
		return cities;
	}

	/**
	 * A gettor using lazy initialization - will create the list if it doesn't exist yet.
	 * 
	 * @return	a Map of the states, sorted alphabetically by name
	 */
	public SortedMap getStateFactors() {
		if (states == null)
		{
			loadRegionAdjustmentFactors();
		}
		
		return states;
	}

	/**
	 * A gettor using lazy initialization - will create the list if it doesn't exist yet.
	 * 
	 * @return	a Map with all the questions that could be asked
	 */
	public Map getQuestions() {
		if (questions == null)
		{
			loadQuestions();
		}
	
		return questions;	
	}

	/**
	 *	Loads the questions from the database.
	 * 
	 *  @return		an empty string if successful, error message otherwise. 
	 */
	private String loadQuestions() {
		Connection conn = null;
		ResultSet rs = null;
		Statement stmt = null;
		String error = "";
		
		try {
			conn = getConnection();
			stmt = conn.createStatement();
			String query = stmtGetQuestions;
			
			logger.debug("About to execute : " + query);
			rs = stmt.executeQuery(query);
	    		
			logger.debug("Finished query");
	    		
			if (rs.next()) {
				logger.debug("Parsing results");
				setQuestions(rs);
				logger.debug("Finished parsing results");
			}
			else {
				error = "Questions could not be loaded from the database.";
			}
		}
		catch(SQLException e) {
            logger.log(Level.WARN, "loadQuestions() Exception", e);
			error = "SQL ERROR in loadQuestions(): " + e.getMessage();
		}
		catch(Exception ex) {
            logger.log(Level.WARN, "loadQuestions() Exception", ex);
			error = "ERROR in loadQuestions(): " + ex.getMessage();
		}
		finally {
			DbUtil.cleanup(conn, stmt, rs);
		}
	
		if (error != "")
		{
			logger.debug("ERROR: "+ error);
		}

		return error;
	}
	
	/**
	 * Parses the resultSet to create a map containing all 
	 * the prototype questions, indexed by name.
	 * 
	 * @param rs	the result set acquired in <code>loadQuestions</code>
	 */
	private void setQuestions(ResultSet rs) throws SQLException {
		String text;
		String name;
		String helpfile;
	
		questions = Collections.synchronizedMap(new HashMap());
	
		do{
			text = StringUtils.trimToEmpty(rs.getString("text"));
			name = StringUtils.trimToEmpty(rs.getString("name"));
			helpfile = StringUtils.trimToEmpty(rs.getString("helpfile"));
		
			logger.debug( "Adding " + name + " with text " + text + " and helpfile " + helpfile);
		
			questions.put(name, new Question(name, text, helpfile));
					
		} while (rs.next());
		
		logger.debug("Added " + String.valueOf(questions.size() + " questions."));
	
	}

	/** 
	 * Reads answers from the database
	 * 
	 * @param method	which tree to retrieve answers for
	 * @return			a <code>Map</code> of vectors which contain answers. Keyed by the name of the <code>Question</code>
	 * 					to which they are answers. 
	 */
	public  Map getAnswers( String method )
	{
		Map answers;
		Map questions = getQuestions();
		
		answers = new HashMap();
		
		Connection conn = null;
		ResultSet rs = null;
		Statement stmt = null;
		String error = "";
		
		try 
		{
			conn = getConnection();
			
			Iterator i = questions.keySet().iterator();
			while (i.hasNext())
			{
				String questionName = (String)i.next();

				// String stmtGetAnswersForQuestion =  "select unique multiChoiceAnswer.text, nodeMultiChoiceAnswerIntersect.sortOrder from nodeMultiChoiceAnswerIntersect, multiChoiceAnswer, question where multiChoiceAnswer.questionId=question.id AND question.name=? AND nodeMultiChoiceAnswerIntersect.methodId=? AND nodeMultiChoiceAnswerIntersect.answerId=multiChoiceAnswer.id ORDER BY nodeMultiChoiceAnswerIntersect.sortOrder ASC";
				// This is the MySQL version
				String stmtGetAnswersForQuestion =  
					"select distinct multiChoiceAnswer.text, nodeMultiChoiceAnswerIntersect.sortOrder from nodeMultiChoiceAnswerIntersect, multiChoiceAnswer, question where multiChoiceAnswer.questionId=question.id AND " + 
					"question.name='" + questionName + "' AND nodeMultiChoiceAnswerIntersect.methodId=" + method + 
					" AND nodeMultiChoiceAnswerIntersect.answerId=multiChoiceAnswer.id ORDER BY nodeMultiChoiceAnswerIntersect.sortOrder ASC";
				stmt = conn.createStatement();
				logger.debug("About to execute : -" + stmtGetAnswersForQuestion + "-   " + questionName);
				rs = stmt.executeQuery(stmtGetAnswersForQuestion);
				
				if (rs.next()) {
					logger.debug("Parsing results");
					Vector newAnswers = parseAnswers(rs);
					answers.put(questionName, newAnswers);
					logger.debug("Added " + newAnswers.size() + " answers to question " + questionName +".");
				}
				else {
					// THis isn't neccessarially an error
					//error = error  + "\n" + "Answers could not be loaded from the database for question " + questionName;
				}
				
				DbUtil.cleanup(null, stmt, rs);
			}
		}
		catch(SQLException e) {
            logger.log(Level.WARN, "getAnswers() Exception", e);
			error = "SQL ERROR in loadQuestions(): " + e.getMessage();
		}
		catch(Exception ex) {
            logger.log(Level.WARN, "getAnswers() Exception", ex);
			error = "ERROR in loadQuestions(): " + ex.getMessage();
		}
		finally {
			DbUtil.cleanup(conn, stmt, rs);
		}

		if (error != "")
		{
			logger.error("ERROR: "+ error);
		}

		return answers;
	}

	/**
	 * Parses a result set to create a vector of answers.
	 * 
	 * @param rs	the result set created in <code>getAnswers</code>
	 * @return		a <code>Vector</code> containing answer objects
	 */
	private Vector parseAnswers(ResultSet rs) throws SQLException
	{
		String text;
	
		Vector answers = new Vector();
		
		do{
			text = StringUtils.trimToEmpty(rs.getString("text"));
		
			logger.debug( "Adding answer \"" + text + "\"");
		
			answers.add(new Answer(text));
					
		} while (rs.next());
		
		logger.debug("Added " + String.valueOf(answers.size() + " answers."));
		
		return answers;
	}
}
