package gov.fema.web.srce;

import java.util.Map;
import java.util.SortedMap;
import java.util.Vector;

import org.apache.log4j.Logger;

import gov.fema.web.srce.objects.DecisionTree;
import gov.fema.web.srce.objects.ExpertDecisionTree;

/**
 * A utility class to hold constants as static variables. This class does no database
 * access - it relies on the ConstantsManager for that.
 *
 * 10-21-04 - Added the questions object and supporting code.
 * 
 * @author Neil Chilson
 */
public class SRCEConstants {

	public static final double FACTOR_2002 = 1.25283; // used to factor up the 1993 costs in the database to 2002 dollars
	public static final int BAD_ID	 = -1;	
	public static final int OTHER = 1;
	
	//	this should be put in a config file. Used from JSPs 
	public static final String helpDir = "/help"; 
	
	public static SortedMap states; // contains the multiplicative factors for each state.
	public static SortedMap cities; // contains the multiplicative factors for each city.
	public static Map questions;    // contains all the questions used in the applications
	
	public static DecisionTree smallStandardTree;
	public static DecisionTree largeStandardTree;
	public static DecisionTree smallExpertTree;
	public static DecisionTree largeExpertTree;
	
	protected static final Logger logger = org.apache.log4j.Logger.getLogger(SRCEConstants.class);
	
	static {
		ConstantsManager cm = new ConstantsManager();
		
		states = cm.getStateFactors();
		cities = cm.getCityFactors();
		questions = cm.getQuestions();
		
		loadDecisionTrees();
				
		cm = null;
	}
	
	/**
	 * Returns the factor by which prices must be multiplied.
	 * 
	 * @param location	 either a state or city name.
	 * @return			 a <code>double</code> containing the multiplication factor.
	 * @throws Exception
	 */
	public static double getLocationMultiplier(String location) throws Exception
	{
		Double result = null;
		
		if (states.containsKey(location))
		{
			result = (Double)states.get(location);
			logger.debug(location + " is a state with a multiplier" + result.doubleValue());
		}
		else if (cities.containsKey(location))
		{
			result = (Double)cities.get(location);
			logger.debug(location + " is a city with a multiplier" + result.doubleValue());
		}
		else
		{
			throw new Exception("Unknown location name");
		}
		
		if (result == null)
		{
			throw new Exception("Unknown location name");
		}
		
		return result.doubleValue();
	}
	
	/**
	 * Default Constructor
	 *
	 */
	public SRCEConstants() {
		super();
	}

	/**
	 * @return	a <code>Vector</code> containing the state names.
	 */
	public Vector getStateNames() 
	{
		return new Vector(SRCEConstants.states.keySet());
	}

	/**
	 * @return	a <code>Vector</code> containing the city names.
	 */
	public Vector getCityNames() 
	{
		return new Vector(SRCEConstants.cities.keySet());
	}
	/**
	 * @return	a <code>Map</code> containing all the questions in the application
	 */
	public static Map getQuestions() {
		return questions;
	}

	/**
	 * A simple gettor for the four types of decisions trees.
	 * 
	 * @param method	which method tree has
	 * @param mode		expert tree or standard tree?
	 * @return			a decision tree, or null
	 */
	public static DecisionTree getDecisionTree(String method, String mode)
	{
		if (method.equals(DecisionTree.METHOD_1) && mode.equals(DecisionTree.STANDARD_MODE))
		{
			return smallStandardTree;			
		}		
		
		if (method.equals(DecisionTree.METHOD_1) && mode.equals(DecisionTree.EXPERT_MODE))
		{
			return smallExpertTree;			
		}
		
		if (method.equals(DecisionTree.METHOD_2) && mode.equals(DecisionTree.STANDARD_MODE))
		{
			return largeStandardTree;			
		}
							
		if (method.equals(DecisionTree.METHOD_2) && mode.equals(DecisionTree.EXPERT_MODE))
		{
			return largeExpertTree;			
		}
		
		logger.error("ERROR in getDecisionTree. Unrecognized Method (" + method + ") or Mode (" + mode + ").");
		
		return null;
	}
	
	/**
	 * This function overloads the get decision tree method to fit a call from Initialize Action
	 * 
	 * @param method
	 * @param isExpertMode
	 * @return 	the proper decision tree
	 */
	public static DecisionTree getDecisionTree(String method, boolean isExpertMode) {
		if (isExpertMode)
		{
			return getDecisionTree(method, DecisionTree.EXPERT_MODE);
		}
		else
		{
			return getDecisionTree(method, DecisionTree.STANDARD_MODE);
		}
	}
	
	/**
	 * Loads the decision trees, using the TreeManager, into static variables
	 * 
	 */
	private static void loadDecisionTrees() 
	{
		TreeManager tm = new TreeManager();
		
		smallStandardTree = tm.createStandardModeTree(DecisionTree.METHOD_1);
		smallExpertTree = tm.createExpertModeTree(DecisionTree.METHOD_1);
		
		largeStandardTree = tm.createStandardModeTree(DecisionTree.METHOD_2);
		largeExpertTree = tm.createExpertModeTree(DecisionTree.METHOD_2);
	}
	
	/*  */
	/**
	 * Added only because the help.jsp needs to access a large expert tree to build itself
	 * 
	 * @return	a large expert decision tree
	 */
	public ExpertDecisionTree getLargeExpertDecisionTree()
	{
		return (ExpertDecisionTree)largeExpertTree;
	}
}
