/*
 * Created on Jul 10, 2003
 *
 */
package gov.fema.web.srce.objects;


import java.util.Vector;

import org.apache.log4j.Logger;

/**
 *	This class is the heart of the decision making process.
 *  Each object of this class has 
 *   Question - Used to display the proper question text, and 
 *              used as a key to match user input to the correct question to test against.
 * 
 *   Answers - Contains possibleAnswers (used to fill out the JSP) and
 *             yesAnswers (Used to decide which direction to go in the tree).
 * 
 *   yesChild - The node to go to if an incoming answer is a yes answer
 *   
 *   noChild - The node to go to if the incoming answer is not a yes answer.
 *
 *  Each node also contains the data attributes from its parent Class, TreeNode
 * 
 * @author n-chilson
 */
public class DecisionNode extends TreeNode{
	
	protected static final Logger logger = org.apache.log4j.Logger.getLogger(DecisionNode.class);
	
	private Question question;
	private TreeNode yesChild;
	private TreeNode noChild;
	private Answers  answers;
	
     /**
      * Constructor for multiple yes Answers
      */	
	 public DecisionNode(int id, 
						 Question newQuestion,
						 Answers answers, 
						 int numBuildings, 
						 double median)
	 {
		 super(id,numBuildings, median);
		 
		 if( newQuestion == null)
		 {
		 	logger.error( "ERROR: newQuestion is null");
		 }
		 
		 this.question = newQuestion;
		 this.answers = answers;
	 }
		
 	/**
 	 * Constructor for simple two answer yes/no node
 	 * 
 	 * @param id
 	 * @param newQuestion
 	 * @param yesAnswerText - used to create an Answer object
 	 * @param noAnswerText  - used to create an Answer object
 	 * @param numBuildings  - data for this node
 	 * @param median        - data for this node
 	 */ 
	public DecisionNode(int id, 
						Question newQuestion,
						String yesAnswerText,
						String noAnswerText,
						int numBuildings,
						double median) 
	{
		super(id, numBuildings, median);
		
		Vector possibleAnswers = new Vector();
		Answer yesAnswer = new Answer(yesAnswerText);
		Answer noAnswer = new Answer(noAnswerText);

		possibleAnswers.add(yesAnswer);
		possibleAnswers.add(noAnswer);
		
		Vector yesAnswers = new Vector();
		yesAnswers.add(yesAnswer);
		
		answers = new Answers(possibleAnswers, yesAnswers);

		question = newQuestion;
	}
	
	
	/**
	 * 
	 * Matches 'answer' with possibleAnswers and yesAnswers
	 * to see which child should be returned.
	 * 
	 * @param answer - an answer matching this node's Question's possible answers.
	 * @return - A TreeNode, either yesChild or noChild, depending on
	 *           the answer passed in.
	 */
	public TreeNode answerQuestion(Answer answer)
	{
		logger.debug( "Answering Question in node " + Integer.toString(getId()));
		if ( answers.isYesAnswer(answer))
		{
			logger.error( "Returning yes child");
			return yesChild;	
		}
		else if ( answer.equals( Answer.Unknown()))
		{
			logger.debug( "Answer is blank.");
			return null;	
		}
		else if( answers.isPossibleAnswer(answer) )
		{
			logger.debug( "Returning no child");
			return noChild;
		}
		else
		{
			logger.debug(  "Invalid choice '" + answer.getText() + "'. Valid choices are: ");
			logger.debug(  answers.getPossibleAnswers().toString());
				
			return null;
		}
	}
	
	/**
	 * @return TreeNode
	 */
	public TreeNode getYesChild()
	{
		return yesChild;
	}

	/**
	 * @return TreeNode
	 */
	public TreeNode getNoChild()
	{
		return noChild;
	}
	
	/**
	 * @return Question
	 */
	public Question getQuestion() {
		return question;
	}

	/**
	 * @return Answers
	 */
	public Answers getAnswers() {
		return answers;
	}
	
	/**
	 * Facade for answers.getPossibleAnswers
	 * @return Vector
	 */
	public Vector getPossibleAnswers()
	{
		return answers.getPossibleAnswers();
	}

	/**
	 * Facade for answers.getYesAnswers
	 * @return Vector
	 */
	public Vector getYesAnswers()
	{
		return answers.getYesAnswers();
	}
	
	/**
	 * Settor for yesChild
	 * @param node
	 */
	public void setYesChild(TreeNode node) {
		yesChild = node;
		yesChild.setParent((TreeNode)this);
	}

	/**
	 * Settor for noChild
	 * @param node
	 */
	public void setNoChild(TreeNode node) {
		noChild = node;
		noChild.setParent((TreeNode)this);
	}
	
	/**
	 * Implementation of an abstract method in TreeNode.
	 * By definition, a decision node cannot be a terminal node.
	 * 
	 * @return boolean
	 */
	public boolean isTerminalNode() {
		return false;
	}
	
	/**
	 * @return int
	 */
	public int getDisplayId() {
		return getId();
	}

}
