/*
 * Created on Jul 10, 2003
 *
 */
package gov.fema.web.srce.objects;

import gov.fema.web.srce.BuildingManager;

import java.util.Map;
import java.util.Vector;
import java.util.HashMap;

/**
 * The <tt>Decision Tree</tt> class is the basic tree used to answer
 * node questions one at a time.
 * 
 * @author n-chilson
 * @see ExpertDecisionTree
 */
public class DecisionTree {
	public static final String METHOD_1 		= "1";
	public static final String METHOD_2 		= "2";
	public static final String METHOD_1_DESCRIPTION = "Limited";
	public static final String METHOD_2_DESCRIPTION = "Detailed";
	public static final String STANDARD_MODE 	= "Interactive";
	public static final String EXPERT_MODE 		= "Form Entry";
		
	private Map treeNodes;
		
	protected String method;
	protected TreeNode root = null;
	protected String mode;
	
	/**
	 * @return The method this tree captures. Either "1" or "2".
	 */	
	public String getTreeMethod()
	{
		return method;
	}
	
	
	/**
	 * @return 	The description of the method this tree uses. 
	 * 			Either "Limited" or "Detailed"
	 */
	public String getTreeMethodDescription()
	{
		if (method.equals(METHOD_1))
		{
			return METHOD_1_DESCRIPTION;
		}
		else if (method.equals(METHOD_2))
		{
			return METHOD_2_DESCRIPTION;
		}
		else
		{
			// error
			return null;
		}
	}
	
	/**
	 * @return 	The mode of traversal that this tree uses. 
	 * 			Either "Interactive" or "Form Entry"
	 */
	public String getTreeMode()
	{
		return mode;
	}
		
	/**
	 * Default constructor
	 */		
	public DecisionTree()
	{
		root = null;
		treeNodes = new HashMap();
	}

	/**
	 * This constructor accepts pre-assembled nodes for the tree, 
	 * and a root indicating the base of the tree. <tt>TreeManager</tt> actually
	 * assembles the trees and calls this method.
	 * 
	 * @param method Which method this tree uses. Either "1" or "2"
	 * @param root The root node of the tree.
	 * @param nodes A collection of nodes that make up the tree.
=	 */
	public DecisionTree(String method, TreeNode root, Map nodes)
	{
		// default tree is not an expert tree.
		mode = STANDARD_MODE;
		 
		this.method = method;
		this.root = root;
		this.treeNodes = nodes;
	}

    /**
     * Adds a node to the treeNodes collection. Note that this 
     * does NOT connect the node to other nodes.
     * @param newNode The node to add to tree 
     */
	protected void addNode(TreeNode newNode)
	{
		treeNodes.put(new Integer(newNode.getId()),newNode);
	}
	
	/**
	 * This function is used to retrieve a specific node from the tree.
	 * @param nodeId The id of the node to retrieve, as a raw int.
	 * @return - A <tt>TreeNodeM</tt> corresponding to the nodeId
	 * @see TreeNode
	 */
	public TreeNode getNode(int nodeId)
	{
		return getNode(new Integer(nodeId));
	}

	/** 
	 * Returns the <tt>TreeNode</tt> in the collection that is indexed 
	 * by the nodeId. Used by the public function getNode(int nodeId),
	 * since maps can only be indexed by objects.
	 * 
	 * @param nodeId an <tt>Integer</tt> object
	 * @return - the <tt>TreeNode</tt> with the key nodeId
	 */
	private TreeNode getNode(Integer nodeId)
	{
		return (TreeNode) treeNodes.get(nodeId);
	}
	
	/**
	 * 
	 * @return - The root <tt>TreeNode</t> of the tree.
	 */
	public TreeNode getRoot() 
	{
		return root;
	}
		
	/** 
	 * 
	 * @return - the <tt>Map</tt> of all the TreeNodes, keyed by nodeId.
	
	public Map getTreeNodes() 
	{
		return treeNodes;
	}
	*/
	
	/**
	 * Returns a <tt>Vector</tt> containing the ancestors of the node 
	 * represented by nodeID
	 * 
	 * @param	nodeID The raw int nodeID to get the ancestors of
	 * @return	A Vector containing the TreeNodes between the root and nodeID
	 */
	public Vector collectParentNodes(int nodeID)
	{
		
		Vector results = new Vector();
	
		TreeNode temp = getNode(nodeID).getParent();
		// Walk tree from current node back to root		 
		while (temp != null)
		{
			results.add(0,temp);
			temp = temp.getParent(); 		 
		}
	
		return results;
	}
	
	/**
	 * Retrieves the node at <strong>questionNodeID</strong> and passes <strong>answer</strong> to it, 
	 * getting back the proper child.
	 * 
	 * @param questionNodeID The id of a node of this tree.
	 * @param answer The answer to try
	 * @return The resulting TreeNode, or <strong>null</strong>.
	 */
	public TreeNode answerQuestion(int questionNodeID, Answer answer)
	{
		TreeNode tempNode = (TreeNode) treeNodes.get(new Integer(questionNodeID));
		
		if (tempNode.isTerminalNode())
		{
			return null;
		}
		else
		{
			return ((DecisionNode)tempNode).answerQuestion(answer);
		}
	}	
	
	
	/**
	 * Initialized a BuildingManager with the data from the 
	 * buildings selected.
	 * 
	 * @param nodeId The raw int id opf the node whose buildings
	 *        we want in the <tt>BuildingManager</tt>.
	 * @return An initialized <tt>BuildingManager</tt>
	 * @see gov.fema.web.srce.BuildingManager
	 */
	public BuildingManager getBuildingsForNode(int nodeId) 
	{
		BuildingManager bm = new BuildingManager();

		if ( getNode(nodeId).isTerminalNode())
		{
			bm.loadBuildingsForTerminalNode(method, nodeId);
		}
		else
		{
			bm.loadBuildingsForInternalNode(method, nodeId);
		}

		return bm;
	}
	
	
	/**
	 * 
	 * @return true if standard mode - false otherwise.
	 */
	public boolean isStandardMode()
	{
		return this.getTreeMode().equals(STANDARD_MODE);	
	}
	
	/**
	 * 
	 * @return true if expert mode - false otherwise.
	 */
	public boolean isExpertMode()
	{
		return this.getTreeMode().equals(EXPERT_MODE);	
	}
			
	/**
	 * 
	 * @return true if small tree - false otherwise.
	 */
	public boolean isMethod_1()
	{
		return this.getTreeMethod().equals(METHOD_1);	
	}
	
	/**
	 * 
	 * @return true if large tree - false otherwise.
	 */		
	public boolean isMethod_2()
	{
		return this.getTreeMethod().equals(METHOD_2);	
	}
}
