/*
 * Created on Jul 15, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package gov.fema.web.srce.objects;

import gov.fema.web.srce.Formatter;
import gov.fema.web.srce.SRCEConstants;

/**
 * Subclass of TreeNode for leaf nodes.
 * 
 * TerminalNodes contain the final results of the SRCE process.
 * This class provides a way to store that information and several ways
 * to extract that info in a pretty format.
 * 
 * @author Neil Chilson
 */
public class TerminalNode extends TreeNode {
	private int terminalNodeId;
	private double q1;
	private double q3;
	private double qr;

	/**
	 * Default Constructor.
	 * 
	 * @param id 				a unique number between 1 and the total number of nodes
	 * @param terminalNodeId	a number between 1 and the number of terminal nodes 
	 * @param numBuildings		the number of buildings represented by this node 
	 * @param median			the raw median of the rehabilitation cost for buildings represented by this node
	 * @param q1				first quartile mark of the rehabilitation cost
	 * @param q3				third quartile mark of the rehabilitation cost
	 * @param qr				ratio of q1 to q3. Gives some idea of the distribution of costs.
	 */
	public TerminalNode(	int id, 
							int terminalNodeId, 
							int numBuildings, 
							double median, 
							double q1, 
							double q3, 
							double qr) {
		super(id, numBuildings, median);

		this.terminalNodeId = terminalNodeId;
		this.q1 = q1;
		this.q3 = q3;
		this.qr = qr;
	}

	/* (non-Javadoc}
	 * @see cart.TreeNode#isTerminalNode()
	 */
	public boolean isTerminalNode() {
		return true;
	}
	
	/**
	 * @return -  the first quartile cost of rehabilitation
	 */
	public double getQ1() {
		return q1;
	}
	
	/**
	 * Multiplies the first quartile cost of rehab by a location multiplier.
	 * 
	 * @param location	the name of the current location. See SRCEConstants for a list of valid locations  
	 * @return	the adjusted first quartile. 
	 * @throws	Exception
	 */
	public double getLocationAdjustedQ1(String location) throws Exception 
	{
		double locationFactor = SRCEConstants.getLocationMultiplier(location);		
		
		return q1 * locationFactor;
	}

	/**
	 * Multiplies the first quartile cost of rehab by a location multiplier
	 * and then formats it as currency 
	 * 
	 * @param location 	the name of the current location. See SRCEConstants for a list of valid locations
	 * @return	a String containing the formatted, adjusted value.
	 * @throws	Exception
	 */
	public String getLocationAdjustedFormattedQ1(String location) throws Exception 
	{
		return Formatter.formatMonetaryValue(this.getLocationAdjustedQ1(location));
	}

	/**
	 * @return	the third quartile cost of rehabilitation
	 */
	public double getQ3() {
		return q3;
	}
	
	/**
	 * Multiplies the third quartile cost of rehab by a location multiplier.
	 * 
	 * @param location	the name of the current location. See SRCEConstants for a list of valid locations  
	 * @return	the adjusted first quartile. 
	 * @throws	Exception
	 */
	public double getLocationAdjustedQ3(String location) throws Exception 
	{
		double locationFactor = SRCEConstants.getLocationMultiplier(location);		
	
		return q3 * locationFactor;
	}

	/**
	 * Multiplies the third quartile cost of rehab by a location multiplier
	 * and then formats it as currency 
	 * 
	 * @param location	the name of the current location. See SRCEConstants for a list of valid locations
	 * @return	a String containing the formatted, adjusted value.
	 * @throws	Exception
	 */
	public String getLocationAdjustedFormattedQ3(String location) throws Exception 
	{
		return Formatter.formatMonetaryValue(this.getLocationAdjustedQ3(location));
	}

	/**
	 * @return	the ratio of the third quartile to the first quartile.
	 */
	public double getQr() {
		return qr;
	}

	/**
	 * Multiplies the quartile ratio by a location multiplier.
	 * 
	 * @param 	location	the name of the current location. See SRCEConstants for a list of valid locations  
	 * @return				the adjusted quartile ratio. 
	 * @throws	Exception
	 */
	public double getLocationAdjustedQr(String location) throws Exception 
	{
		double locationFactor = SRCEConstants.getLocationMultiplier(location);		
	
		return qr * locationFactor;
	}

	/**
	 * Multiplies the quartile ratio by a location multiplier
	 * and then formats it as currency 
	 * 
	 * @param location	the name of the current location. See SRCEConstants for a list of valid locations
	 * @return	a String containing the formatted, adjusted value.
	 * @throws	Exception
	 */
	public String getLocationAdjustedFormattedQr(String location) throws Exception 
	{
		return Formatter.formatDecimalValue(this.getLocationAdjustedQr(location));
	}
	
	/**
	 * @return	the identifier of this terminal node
	 */
	public int getTerminalNodeId() {
		return terminalNodeId;
	}

	/* (non-Javadoc)
	 * @see gov.fema.web.srce.TreeNode#getDisplayId()
	 */
	public int getDisplayId() {
		return getTerminalNodeId();
	}
}
