/*
 * Created on Jul 15, 2003
 *
 */
package gov.fema.web.srce.objects;

import org.apache.log4j.Logger;

import gov.fema.web.srce.Formatter;
import gov.fema.web.srce.SRCEConstants;

/**
 * @author n-chilson
 *
 * Parent of DecisionNode and TerminalNode.
 * 
 */
public abstract class TreeNode implements Comparable{
	private static final int BAD_ID =-1;
	
	protected static final Logger logger = org.apache.log4j.Logger.getLogger(TreeNode.class);
	
	private TreeNode parent;
	private int id;
	private int numBuildings;
	private double median;
	
	/**
	 * Overriden by subclasses to indicate the type of the node.
	 * 
	 * @return boolean
	 */
	abstract public boolean isTerminalNode();
	
	/**
	 * Display ID differs from the node id for terminal nodes
	 * but not for decision nodes. Therefore, this should be overridden in
	 * each subclass to provide proper behavior.
	 * 
	 * @return - This node's display ID. 
	 */
	abstract public int getDisplayId();
	
	/**
	 * Relys on the abstract isTerminalNode() function.
	 * 
	 * @return boolean
	 */
	public boolean isDecisionNode()
	{
		return !isTerminalNode();
	}
	
	/**
	 * Default Constructor.
	 *
	 */
	public TreeNode()
	{
		super();
		id = BAD_ID;
	}
		
	/**
	 * Subclasses should call this as part of their constructors.
	 * 
	 * @param id - identifier for this node.
	 */
	public TreeNode(int id)
	{
		super();
		this.id = id;
	}
	
	/**
	 * This constructor initializes the node completely except for setting
	 * parent.
	 * 
	 * @param id	- Id for this node
	 * @param numBuildings - number of buildings whose data was used to create this node.
	 * @param median - median cost of rehabilitation for all buildings related to this node.
	 */
	public TreeNode(int id, int numBuildings, double median)
	{
		super();
		this.id = id;
		this.numBuildings = numBuildings;
		this.median = median;
	}
	
	/**
	 * 
	 * @return - parent of this node.
	 */
	public TreeNode getParent() {
		return parent;
	}

	/**
	 * @param node - this node's new parent.
	 */
	public void setParent(TreeNode node) {
		parent = node;
	}

	/**
	 * @return - identifier for this node.
	 */
	public int getId() {
		return id;
	}

	/**
	 * @return - median cost of rehabilitation for all buildings related to this node.
	 */
	public double getMedian() {
		return median;
	}

	/**
	 * Formats median as a monetary value.
	 * 
	 * @return - a string containg the formatted median.
	 */
	public String getFormattedMedian()
	{
		return Formatter.formatMonetaryValue(this.getMedian()); 
	}


	/**
	 * Multiplies the median by the location multiplier.
	 * 
	 * @param location - Current location. See SRCEConstants for a list of valid locations.
	 * @return - a double containing the adjusted median.
	 * @throws Exception
	 */
	public double getLocationAdjustedMedian(String location) throws Exception
	{
		double locationFactor = SRCEConstants.getLocationMultiplier(location);
		logger.debug( "Multiplying " + median + " * " + locationFactor + " = " + median * locationFactor);
		return median * locationFactor;
	}

	/**
	 * Multiplies the median by the location multiplier and then formats it
	 * as a monetary value.
	 * 
	 * @param location - Current location. See SRCEConstants for a list of valid locations.
	 * @return - a string containing the adjusted median formatted as currency.
	 * @throws Exception
	 */
	public String getLocationAdjustedFormattedMedian(String location) throws Exception
	{
		return Formatter.formatMonetaryValue(this.getLocationAdjustedMedian(location));
	}

	/**
	 * @return - number of buildings whose data resulted in this node.
	 */
	public int getNumBuildings() {
		return numBuildings;
	}

	/**
	 * @param median - the average cost of rehabilitation for all buildings that reach this node.
	 */
	public void setMedian(double median) {
		this.median = median;
	}

	/**
	 * @param numBuildings - number of buildings whose data resulted in this node.
	 */
	public void setNumBuildings(int numBuildings) {
		this.numBuildings = numBuildings;
	}

	/* (non-Javadoc)
	 * 
	 * This was added so that we could use a node as a key in the PreviousAnswers object.
	 * It compares nodes purely based on their id.
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object obj) {
		// simple optimization checks to see if they are the same object
		if (this == obj) return 0;
		
		//Performing explicit checks for nullity and type are made
	    //redundant by the following cast, which will throw
	    //NullPointerException and ClassCastException in these
	    //respective cases.
	    final TreeNode that = (TreeNode) obj;
	    
	    Integer thisId = new Integer (this.id);
	    Integer thatId = new Integer (that.id);
	    
		return thisId.compareTo(thatId);
	}
	
}
